"""
Tests for state messages
"""

import pytest
from mqtt_protocol import (
    StationStateMessage,
    StationInfo,
    RobotStateMessage
)
from mqtt_protocol.utils.constants import (
    STATION_MODE_OPERATION,
    DEVICE_STATE_NORMAL,
    DEVICE_STATE_ALARM
)


def test_station_state_message():
    # Test creation
    state_msg = StationStateMessage.create(
        state=STATION_MODE_OPERATION,
        smoke="1,1,1,1",
        fire=DEVICE_STATE_NORMAL,
        temp=25,
        humid=60,
        totalElect=1000.5
    )

    # Test serialization
    json_data = state_msg.to_json()
    assert isinstance(json_data, str)

    # Test deserialization
    parsed_msg = StationStateMessage.from_json(json_data)
    assert parsed_msg.state == STATION_MODE_OPERATION
    assert parsed_msg.temp == 25
    assert parsed_msg.humid == 60
    assert parsed_msg.totalElect == 1000.5


def test_station_info():
    info = StationInfo(
        name="Test Station",
        stationID="100001",
        version="V2.6.2",
        startDate="2024-02-24",
        robotNum=2,
        robotID="1234567890123456,1234567890123457",
        chgNum=8,
        chgID="CHG001,CHG002,CHG003,CHG004,CHG005,CHG006,CHG007,CHG008",
        laneType=1,
        rateModelID="RATE001",
        chgVersion="V1.0"
    )

    assert info.name == "Test Station"
    assert info.stationID == "100001"
    assert info.robotNum == 2
    assert info.chgNum == 8


def test_station_state_validation():
    # Test invalid state
    with pytest.raises(ValueError):
        StationStateMessage.create(
            state=999,  # Invalid state
            smoke="1,1,1,1",
            fire=DEVICE_STATE_NORMAL,
            temp=25,
            humid=60,
            totalElect=1000.5
        )

    # Test invalid smoke detector format
    with pytest.raises(ValueError):
        StationStateMessage.create(
            state=STATION_MODE_OPERATION,
            smoke="invalid",
            fire=DEVICE_STATE_NORMAL,
            temp=25,
            humid=60,
            totalElect=1000.5
        )


def test_robot_state():
    robot_msg = RobotStateMessage(
        robotID="1234567890123456",
        state=1,
        fault=None,
        runMode=1
    )

    assert robot_msg.robotID == "1234567890123456"
    assert robot_msg.state == 1
    assert robot_msg.fault is None
    assert robot_msg.runMode == 1


def test_robot_state_with_fault():
    robot_msg = RobotStateMessage(
        robotID="1234567890123456",
        state=3,  # Fault state
        fault="ERROR001,ERROR002",
        runMode=1
    )

    assert robot_msg.state == 3
    assert robot_msg.fault == "ERROR001,ERROR002"